"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.scrapeHtmlWeb = void 0;
const axios_1 = __importDefault(require("axios"));
const cheerio_1 = __importDefault(require("cheerio"));
function scrapeHtmlWeb(options) {
    return __awaiter(this, void 0, void 0, function* () {
        let result = [];
        const { url, mainSelector, childrenSelector, list, proxy } = options || {};
        const proxyURI = proxy ? proxy : "";
        yield (0, axios_1.default)(`${proxyURI}${encodeURIComponent(url)}`)
            .then((response) => {
            const html_data = response.data.contents;
            const $ = cheerio_1.default.load(html_data);
            $(mainSelector)
                .children()
                .each((_, parentElem) => {
                let obj = {};
                childrenSelector.forEach((el) => {
                    const { key, selector, attr, type, canBeEmpty, replace } = el;
                    const value = $(parentElem).find(selector).length
                        ? $(parentElem).find(selector)
                        : $(parentElem);
                    if (!attr && !type) {
                        let text = value.text().trim();
                        if (replace) {
                            if (replace instanceof RegExp) {
                                text = text.replace(replace, "");
                            }
                            else if (typeof replace === "function") {
                                text = replace(text);
                            }
                        }
                        obj = Object.assign(Object.assign({}, obj), { [key]: text });
                    }
                    if (attr && attr !== "") {
                        if (value.attr(attr) || canBeEmpty) {
                            obj = Object.assign(Object.assign({}, obj), { [key]: value.attr(attr) });
                        }
                    }
                    else if (type === "text") {
                        if (value.text().trim() || canBeEmpty) {
                            let text = value.text().trim();
                            if (replace) {
                                if (replace instanceof RegExp) {
                                    text = text.replace(replace, "");
                                }
                                else if (typeof replace === "function") {
                                    text = replace(text);
                                }
                            }
                            obj = Object.assign(Object.assign({}, obj), { [key]: text });
                        }
                    }
                    else if (type === "html") {
                        if (value.prop("outerHTML") || canBeEmpty) {
                            let html = value.prop("outerHTML");
                            if (replace) {
                                if (replace instanceof RegExp) {
                                    if (html) {
                                        html = html.replace(replace, "");
                                    }
                                }
                                else if (typeof replace === "function") {
                                    if (html) {
                                        html = replace(html);
                                    }
                                }
                            }
                            obj = Object.assign(Object.assign({}, obj), { [key]: html });
                        }
                    }
                });
                if (Object.keys(obj).length) {
                    result.push(obj);
                }
                if (list === false) {
                    return false;
                }
            });
        })
            .catch((err) => {
            throw new Error(err);
        });
        return result;
    });
}
exports.scrapeHtmlWeb = scrapeHtmlWeb;
exports.default = {
    scrapeHtmlWeb,
};
